# TODO: Regenerate this file with sql2code
table_definition = {
    'phone_table': {
        'table_definition_id': 1,
        'entity_id': None,
        'schema': 'phone',
        'github_organization': '',
        'github_repo': '',
        'seq': 0,
        'table_list_id': None,
        'is_created_updated_fields': 1,
        'is_logical_delete_fields': 1,
        'profile_mapping_table_id': 2,
        'is_ml_table': None,
        'table_type': None,
        'is_number_column': 1,
        'is_test_data_column': None,
        'is_main_column': None,
        'is_single_and_multi_value': None,
        'name_of_single_value_column': None,
        'is_mapping_table': None,
        'is_entity': None,
        'is_history': None,
    },
    'phone_profile_table': {
        'table_definition_id': 2,
        'entity_id': None,
        'schema': 'phone_profile',
        'github_organization': '',
        'github_repo': '',
        'seq': 0,
        'table_list_id': None,
        'is_created_updated_fields': 1,
        'is_logical_delete_fields': 1,
        'profile_mapping_table_id': None,
        'is_ml_table': None,
        'table_type': None,
        'is_number_column': 0,
        'is_test_data_column': None,
        'is_main_column': None,
        'is_single_and_multi_value': None,
        'name_of_single_value_column': None,
        'is_mapping_table': None,
        'is_entity': None,
        'is_history': None,
    },
    'email_address_profile_table': {
        'table_definition_id': 3,
        'entity_id': None,
        'schema': 'email_address_profile',
        'github_organization': '',
        'github_repo': '',
        'seq': 0,
        'table_list_id': None,
        'is_created_updated_fields': 1,
        'is_logical_delete_fields': 1,
        'profile_mapping_table_id': None,
        'is_ml_table': None,
        'table_type': None,
        'is_number_column': 0,
        'is_test_data_column': None,
        'is_main_column': None,
        'is_single_and_multi_value': None,
        'name_of_single_value_column': None,
        'is_mapping_table': None,
        'is_entity': None,
        'is_history': None,
    },
    'email_address_table': {
        'table_definition_id': 4,
        'entity_id': None,
        'schema': 'email_address',
        'github_organization': '',
        'github_repo': '',
        'seq': 0,
        'table_list_id': None,
        'is_created_updated_fields': 1,
        'is_logical_delete_fields': 1,
        'profile_mapping_table_id': 3,
        'is_ml_table': None,
        'table_type': None,
        'is_number_column': 1,
        'is_test_data_column': None,
        'is_main_column': None,
        'is_single_and_multi_value': None,
        'name_of_single_value_column': None,
        'is_mapping_table': None,
        'is_entity': None,
        'is_history': None,
    },
    'user_table': {
        'table_definition_id': 5,
        'entity_id': None,
        'schema': 'user',
        'github_organization': '',
        'github_repo': '',
        'seq': 0,
        'table_list_id': None,
        'is_created_updated_fields': 1,
        'is_logical_delete_fields': 1,
        'profile_mapping_table_id': 6,
        'is_ml_table': None,
        'table_type': None,
        'is_number_column': 1,
        'is_test_data_column': None,
        'is_main_column': None,
        'is_single_and_multi_value': None,
        'name_of_single_value_column': None,
        'is_mapping_table': None,
        'is_entity': None,
        'is_history': None,
    },
    'profile_user_table': {
        'table_definition_id': 6,
        'entity_id': None,
        'schema': 'profile_user',
        'github_organization': '',
        'github_repo': '',
        'seq': 0,
        'table_list_id': None,
        'is_created_updated_fields': 1,
        'is_logical_delete_fields': 1,
        'profile_mapping_table_id': None,
        'is_ml_table': None,
        'table_type': None,
        'is_number_column': 0,
        'is_test_data_column': None,
        'is_main_column': None,
        'is_single_and_multi_value': None,
        'name_of_single_value_column': None,
        'is_mapping_table': None,
        'is_entity': None,
        'is_history': None,
    },
    'group_profile_table': {
        'table_definition_id': 7,
        'entity_id': None,
        'schema': 'group_profile',
        'github_organization': '',
        'github_repo': '',
        'seq': 0,
        'table_list_id': None,
        'is_created_updated_fields': 1,
        'is_logical_delete_fields': 1,
        'profile_mapping_table_id': None,
        'is_ml_table': None,
        'table_type': None,
        'is_number_column': 0,
        'is_test_data_column': None,
        'is_main_column': None,
        'is_single_and_multi_value': None,
        'name_of_single_value_column': None,
        'is_mapping_table': None,
        'is_entity': None,
        'is_history': None,
    },
    'group_table': {
        'table_definition_id': 8,
        'entity_id': None,
        'schema': 'group',
        'github_organization': '',
        'github_repo': '',
        'seq': 0,
        'table_list_id': None,
        'is_created_updated_fields': 1,
        'is_logical_delete_fields': 1,
        'profile_mapping_table_id': 7,
        'is_ml_table': 1,
        'table_type': None,
        'is_number_column': 1,
        'is_test_data_column': None,
        'is_main_column': None,
        'is_single_and_multi_value': None,
        'name_of_single_value_column': None,
        'is_mapping_table': None,
        'is_entity': None,
        'is_history': None,
    },
    'group_label_table': {
        'table_definition_id': 9,
        'entity_id': None,
        'schema': 'group_label',
        'github_organization': '',
        'github_repo': '',
        'seq': 0,
        'table_list_id': None,
        'is_created_updated_fields': 1,
        'is_logical_delete_fields': 1,
        'profile_mapping_table_id': None,
        'is_ml_table': None,
        'table_type': None,
        'is_number_column': 0,
        'is_test_data_column': None,
        'is_main_column': None,
        'is_single_and_multi_value': None,
        'name_of_single_value_column': None,
        'is_mapping_table': None,
        'is_entity': None,
        'is_history': None,
    },
    'profile_metics_table': {
        'table_definition_id': 10,
        'entity_id': None,
        'schema': 'profile_metics',
        'github_organization': '',
        'github_repo': '',
        'seq': 0,
        'table_list_id': None,
        'is_created_updated_fields': 1,
        'is_logical_delete_fields': 1,
        'profile_mapping_table_id': None,
        'is_ml_table': None,
        'table_type': None,
        'is_number_column': 0,
        'is_test_data_column': None,
        'is_main_column': None,
        'is_single_and_multi_value': None,
        'name_of_single_value_column': None,
        'is_mapping_table': None,
        'is_entity': None,
        'is_history': None,
    },
    'profile_url_table': {
        'table_definition_id': 11,
        'entity_id': None,
        'schema': 'profile_url',
        'github_organization': '',
        'github_repo': '',
        'seq': 0,
        'table_list_id': None,
        'is_created_updated_fields': 1,
        'is_logical_delete_fields': 1,
        'profile_mapping_table_id': None,
        'is_ml_table': None,
        'table_type': None,
        'is_number_column': 0,
        'is_test_data_column': None,
        'is_main_column': None,
        'is_single_and_multi_value': None,
        'name_of_single_value_column': None,
        'is_mapping_table': None,
        'is_entity': None,
        'is_history': None,
    },
    'user_external_table': {
        'table_definition_id': 12,
        'entity_id': None,
        'schema': 'user_external',
        'github_organization': '',
        'github_repo': '',
        'seq': 0,
        'table_list_id': None,
        'is_created_updated_fields': 1,
        'is_logical_delete_fields': 1,
        'profile_mapping_table_id': None,
        'is_ml_table': None,
        'table_type': None,
        'is_number_column': 1,
        'is_test_data_column': None,
        'is_main_column': None,
        'is_single_and_multi_value': None,
        'name_of_single_value_column': None,
        'is_mapping_table': None,
        'is_entity': None,
        'is_history': None,
    },
    'location_table': {
        'table_definition_id': 13,
        'entity_id': None,
        'schema': 'location',
        'github_organization': '',
        'github_repo': '',
        'seq': 0,
        'table_list_id': None,
        'is_created_updated_fields': 1,
        'is_logical_delete_fields': 1,
        'profile_mapping_table_id': 14,
        'is_ml_table': None,
        'table_type': None,
        'is_number_column': 1,
        'is_test_data_column': None,
        'is_main_column': None,
        'is_single_and_multi_value': None,
        'name_of_single_value_column': None,
        'is_mapping_table': None,
        'is_entity': None,
        'is_history': None,
    },
    'location_profile_table': {
        'table_definition_id': 14,
        'entity_id': None,
        'schema': 'location_profile',
        'github_organization': '',
        'github_repo': '',
        'seq': 0,
        'table_list_id': None,
        'is_created_updated_fields': 1,
        'is_logical_delete_fields': 1,
        'profile_mapping_table_id': None,
        'is_ml_table': None,
        'table_type': None,
        'is_number_column': 0,
        'is_test_data_column': None,
        'is_main_column': None,
        'is_single_and_multi_value': None,
        'name_of_single_value_column': None,
        'is_mapping_table': None,
        'is_entity': None,
        'is_history': None,
    },
    'profile_profile_table': {
        'table_definition_id': 15,
        'entity_id': None,
        'schema': 'profile_profile',
        'github_organization': '',
        'github_repo': '',
        'seq': 0,
        'table_list_id': None,
        'is_created_updated_fields': 1,
        'is_logical_delete_fields': 1,
        'profile_mapping_table_id': None,
        'is_ml_table': None,
        'table_type': None,
        'is_number_column': 0,
        'is_test_data_column': None,
        'is_main_column': None,
        'is_single_and_multi_value': None,
        'name_of_single_value_column': None,
        'is_mapping_table': None,
        'is_entity': None,
        'is_history': None,
    },
    'profile_table': {
        'table_definition_id': 16,
        'entity_id': None,
        'schema': 'profile',
        'github_organization': '',
        'github_repo': '',
        'seq': 0,
        'table_list_id': None,
        'is_created_updated_fields': 1,
        'is_logical_delete_fields': 1,
        'profile_mapping_table_id': None,
        'is_ml_table': 1,
        'table_type': None,
        'is_number_column': 1,
        'is_test_data_column': None,
        'is_main_column': None,
        'is_single_and_multi_value': None,
        'name_of_single_value_column': None,
        'is_mapping_table': None,
        'is_entity': None,
        'is_history': None,
    },
    'person_table': {
        'table_definition_id': 17,
        'entity_id': None,
        'schema': 'person',
        'github_organization': '',
        'github_repo': '',
        'seq': 0,
        'table_list_id': None,
        'is_created_updated_fields': 1,
        'is_logical_delete_fields': 1,
        'profile_mapping_table_id': None,
        'is_ml_table': None,
        'table_type': None,
        'is_number_column': 1,
        'is_test_data_column': None,
        'is_main_column': None,
        'is_single_and_multi_value': 1,
        'name_of_single_value_column': 'main_email_address',
        'is_mapping_table': None,
        'is_entity': None,
        'is_history': None,
    },
    'test_mysql_table': {
        'table_definition_id': 18,
        'entity_id': None,
        'schema': 'test',
        'github_organization': '',
        'github_repo': '',
        'seq': 0,
        'table_list_id': None,
        'is_created_updated_fields': 1,
        'is_logical_delete_fields': 1,
        'profile_mapping_table_id': None,
        'is_ml_table': None,
        'table_type': None,
        'is_number_column': 1,
        'is_test_data_column': None,
        'is_main_column': None,
        'is_single_and_multi_value': None,
        'name_of_single_value_column': None,
        'is_mapping_table': None,
        'is_entity': None,
        'is_history': None,
    },
    'event_table': {
        'table_definition_id': 19,
        'entity_id': None,
        'schema': 'event',
        'github_organization': '',
        'github_repo': '',
        'seq': 0,
        'table_list_id': None,
        'is_created_updated_fields': 1,
        'is_logical_delete_fields': 1,
        'profile_mapping_table_id': None,
        'is_ml_table': None,
        'table_type': None,
        'is_number_column': None,
        'is_test_data_column': None,
        'is_main_column': None,
        'is_single_and_multi_value': None,
        'name_of_single_value_column': None,
        'is_mapping_table': None,
        'is_entity': None,
        'is_history': None,
    },
    'logger_table': {
        'table_definition_id': 20,
        'entity_id': None,
        'schema': 'logger',
        'github_organization': '',
        'github_repo': '',
        'seq': 0,
        'table_list_id': None,
        'is_created_updated_fields': 1,
        'is_logical_delete_fields': 1,
        'profile_mapping_table_id': None,
        'is_ml_table': None,
        'table_type': None,
        'is_number_column': None,
        'is_test_data_column': None,
        'is_main_column': None,
        'is_single_and_multi_value': None,
        'name_of_single_value_column': None,
        'is_mapping_table': None,
        'is_entity': None,
        'is_history': 1,
    },
}
